<?php

namespace Drupal\language_country_negotiation\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\language_country_negotiation\Service\CountryManagerInterface;
use Drupal\language_country_negotiation\Service\CountryRepositoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a country switcher block.
 *
 * @Block(
 *   id = "language_country_negotiation_country_switcher",
 *   admin_label= @Translation("Country switcher")
 * )
 */
class CountrySwitcherBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a new CountrySwitcherBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\language_country_negotiation\Service\CountryManagerInterface $countryManager
   *   The country manager.
   * @param \Drupal\language_country_negotiation\Service\CountryRepositoryInterface $countryRepository
   *   The country repository.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Core\Path\PathMatcherInterface $pathMatcher
   *   The path matcher.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected CountryManagerInterface $countryManager,
    protected CountryRepositoryInterface $countryRepository,
    protected LanguageManagerInterface $languageManager,
    protected RequestStack $requestStack,
    protected PathMatcherInterface $pathMatcher,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ): static {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('language_country_negotiation.country_manager'),
      $container->get('language_country_negotiation.country_repository'),
      $container->get('language_manager'),
      $container->get('request_stack'),
      $container->get('path.matcher'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {

    $current_url = $this->pathMatcher->isFrontPage() ? Url::fromRoute('<front>') : Url::fromRoute('<current>');
    $query = $this->requestStack->getCurrentRequest()->query->all();

    $current_langcode = $this->languageManager->getCurrentLanguage(LanguageInterface::TYPE_URL)->getId();
    $native_languages = $this->languageManager->getNativeLanguages();

    $current_country_code = $this->countryManager->getCurrentCountryCode();
    $countries = $this->countryRepository->getCountries();

    // Maybe move to negotiation plugin. Build list of country links.
    $links = [];
    foreach ($countries as $country_code => $country_name) {

      // Resolve target langcode. Either the current language is allowed in
      // the target country or the primary language in the target is used.
      if ($this->countryManager->isLanguageAvailable($country_code, $current_langcode)) {
        $langcode = $current_langcode;
      }
      else {
        $langcode = $this->countryManager->getPrimaryLangcode($country_code);
      }

      // Select the target language object from the native languages.
      if (!$language = $native_languages[$langcode] ?? NULL) {
        continue;
      }

      // Resolve the target URL for this country by passing the country code
      // as an option. The country code option is resolved in the outbound
      // path processing of the language country negotiation plugin.
      $url = clone $current_url;
      $url->setOptions([
        'language' => $language,
        'country_code' => $country_code,
        'query' => $query,
      ]);

      // Generate country url based on current route.
      $links[] = [
        'url' => $url,
        'title' => $country_name,
        'country_code' => $country_code,
        'attributes' => [
          'class' => 'country-link',
          'data-country-code' => $country_code,
        ],
        'is_active' => $country_code === $current_country_code,
        'query' => $query,
      ];
    }

    return [
      '#theme' => 'links__country_switcher_block',
      '#links' => $links,
      '#set_active_class' => TRUE,
      '#attached' => [
        'library' => ['language_country_negotiation/active-link'],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts(): array {
    return Cache::mergeContexts(parent::getCacheContexts(), [
      'url.query_args',
      'url.path',
      'current_country',
    ]);
  }

}
